/*
 * gp_djcon.h --- text console over graphics
 *
 * Copyright (C) 1998  Michael Bukin
 */

#ifndef __bma_gp_djcon_h
#define __bma_gp_djcon_h

/* Console font data.  */
extern char djCON_font_data[256][8];

/*
 * djCON_t holds information about console state.
 * Drivers may extend it with driver state.
 */
#define djCON_common\
  /* Screen width and height.  */\
  int sw, sh;\
  /* Page width and height and visible page height.  */\
  int pw, ph, vph;\
  /* Text width and height and visible text height.  */\
  int tw, th, vth;\
  /* Character width and height.  */\
  int cw, ch;\
  /* Current cursor position.  */\
  int cx, cy;\
  /* Offset of page from top-left corner of screen.  */\
  int ox, oy;\
  /* Movement increment (left-right, up-down).  */\
  int lr, ud;\
  /* Print character in cursor position.  */\
  void (*print_char) (int ch);\
  /* Clear character in cursor position.  */\
  void (*clear_char) (void);\
  /* Blink cursor (swap foreground & background).  */\
  void (*blink_char) (void);\
  /* Scroll text up one line.  */\
  void (*scroll) (void);\
  /* Update the page screen.  */\
  void (*update) (void);\
  /* Update the whole screen.  */\
  void (*toggle) (void);\
  /* Is it safe to call any function?  */\
  int (*is_invalid) (void)

#define djCON_common_data\
  0, 0,\
  0, 0, 0,\
  0, 0, 0,\
  0, 0,\
  0, 0,\
  0, 0,\
  0, 0

/* Console structure (should be extended by drivers).  */
typedef struct djCON_t
{
  djCON_common;
} djCON_t;

/* Hopefully faster version of outportw.  */
extern inline void
dj_outportw (int port, int data)
{
  __asm__ __volatile__ ("outw %w1,%w0"
			:
			: "d" (port), "a" (data));
}

/* Hopefully faster versions of _far functions.  */
extern inline unsigned char
_dj_farpeekb (int selector, unsigned long offset)
{
  unsigned char result;
  __asm__ __volatile__ ("movw %w0,%%fs\n\t"
			".byte 0x64\n\t"
			"movb (%k2),%b0"
			: "=q" (result)
			: "rm" (selector), "r" (offset));
  return result;
}
extern inline void
_dj_farpokeb (int selector, unsigned long offset, unsigned char value)
{
  __asm__ __volatile__ ("movw %w0,%%fs\n\t"
			".byte 0x64\n\t"
			"movb %b1,(%k2)"
			:
			: "rm" (selector), "qi" (value), "r" (offset));
}
extern inline void
_dj_farsetsel (int selector)
{
  __asm__ __volatile__ ("movw %w0,%%fs"
			:
			: "rm" (selector));
}
extern inline unsigned char
_dj_farnspeekb (unsigned long offset)
{
  unsigned char result;
  __asm__ __volatile__ (".byte 0x64\n\t"
			"movb (%k1),%b0"
			: "=q" (result)
			: "r" (offset));
  return result;
}
extern inline void
_dj_farnspokeb (unsigned long offset, unsigned char value)
{
  __asm__ __volatile__ (".byte 0x64\n\t"
			"movb %b0,(%k1)"
			:
			: "qi" (value), "r" (offset));
}

#ifdef __cplusplus
extern "C" {
#endif

  /* Capture console.  */
  void djCON_capture (struct djCON_t* con);
  /* Release console.  */
  void djCON_release (void);
  /* Update console (page).  */
  void djCON_update (void);

#ifdef __cplusplus
}
#endif

#endif /* !__bma_gp_djcon_h */

/*
 * gp_djcon.h ends here
 */
